#include <stdio.h>
#include <stdlib.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"
#include <string.h>

#define SCREEN_WIDTH    400
#define SCREEN_HEIGHT   400

double table[2][2][SCREEN_HEIGHT+2][SCREEN_WIDTH+2];
double prod1Table[2][SCREEN_HEIGHT+2][SCREEN_WIDTH+2];
double prod2Table[2][SCREEN_HEIGHT+2][SCREEN_WIDTH+2];

double DA = 1.0;
double DB = 0.5;
double f = 0.055;
double k = 0.062;
int curT = 0;

void product()
{
    for (int t = 0; t < 2; t++)
    {
        for (int y = 0; y < SCREEN_HEIGHT+2; ++y)
            for (int x = 1; x < SCREEN_WIDTH+1; ++x)
                prod1Table[t][y][x] = table[curT][t][y][x - 1] + 4.0f * table[curT][t][y][x] + table[curT][t][y][x + 1];

        for (int y = 1; y < SCREEN_HEIGHT+1; ++y)
            for (int x = 1; x < SCREEN_WIDTH+1; ++x)
                prod2Table[t][y][x] = prod1Table[t][y - 1][x] + 4.0f * prod1Table[t][y][x] + prod1Table[t][y + 1][x];
    }
}

float tim = 0;
int cnt = 0;
void update()
{
    sys.StartPerfCounter();
    double ca = 1.0 - f - 1.8 * DA;
    double cb = 1.0 - f - 1.8 * DB - k;

    product();

    for (int y = 1; y < SCREEN_HEIGHT+1; ++y)
        for (int x = 1; x < SCREEN_WIDTH+1; ++x)
        {
            double A = table[curT][0][y][x];
            double B = table[curT][1][y][x];

            table[1-curT][0][y][x] = 0.05 * DA * prod2Table[0][y][x] - A * B * B + ca * A + f;
            table[1-curT][1][y][x] = 0.05 * DB * prod2Table[1][y][x] + A * B * B + cb * B;
        }

    curT = 1 - curT;

    cnt++;
    if (cnt < 2000)
        tim += sys.StopPerfCounter();
    else if (cnt == 2000)
        printf("%f\n", tim);
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Reaction/Diffusion 7", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

    // initialize the table
    for (int y = 0; y < SCREEN_HEIGHT+2; ++y)
        for (int x = 0; x < SCREEN_WIDTH+2; ++x)
            for (int t = 0; t < 2; ++t)
            {
                table[t][0][y][x] = 1.0;
                table[t][1][y][x] = 0.0;
            }

    for (int y = 0; y < 10; ++y)
        for (int x = 0; x < 10; ++x)
        {
            int posX = SCREEN_WIDTH  / 2 + x - 4;
            int posY = SCREEN_HEIGHT / 2 + y - 4;
            table[0][1][posY][posX] = 1.0;
        }

    while (sys.isQuitRequested() == false)
    {
        // draw the current table
        for (int y = 0; y < SCREEN_HEIGHT; ++y)
            for (int x = 0; x < SCREEN_WIDTH; ++x)
            {
                int r = table[0][0][y+1][x+1] * 255;
                int b = table[0][1][y+1][x+1] * 255;
                gfx.setPixel(x, y, Color(r, 0, b));
            }

        gfx.render();
        sys.processEvents();

        // update the table
        update();
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
